/***************************************************
 * atmospheric_pressure.pde
 * 
 * エレキジャック（ http://www.eleki-jack.com/FC/2010/07/arduinospi.html）から頂いた秋月の気圧センサSCP1000の制御コードを使って、
 * 計測値をシリアルへ出力しています。
 * Read the Barometric Pressure and Temperature values from SCP1000 sensor
 ************************************************/
#include <SPI.h>

// コード定義
#define TEMPOUT        0x84
#define DATAREAD8_MSB  0x7c  // 上位8bit読み出し
#define DATAREAD16_LSB 0x80  // 上位16bit読み出し
#define DUMMY_DATA     0xFF

/*--------------------------------- 
 センサ初期化
 ---------------------------------*/
void SPI_MasterInit(void)
{
  pinMode(11, OUTPUT);       // MOSI = 11
  pinMode(13, OUTPUT);       // SCK  = 13
  pinMode(10, OUTPUT);       // SS   = 10
  pinMode(12, INPUT);        // MISO = 12
  SPCR = B01010001;
}
/*--------------------------------- 
 コマンドの送信 & 応答の受信 1
 ---------------------------------*/
byte SPIMSTX(byte cData)
{
  SPDR = cData;              // データレジスタへ送信データをセット
  while(!(SPSR & 0x80));     // ステータスレジスタのビット7が0になるのを待つ
  return SPDR;               // 受信したデータを返す
}
/*--------------------------------- 
 コマンドの送信 & 応答の受信 2
 ---------------------------------*/
byte SPIMSTXSP(byte sData)
{
  byte rData;

  Serial.print("sent data = ");  // 送信データを16進数で出力
  Serial.print(sData, HEX);
  rData = SPIMSTX(sData);
  Serial.print(" receive data = ");
  Serial.println(rData, HEX);

  return rData;
}

void setup()
{
  SPI_MasterInit();
  Serial.begin(9600);

  digitalWrite(10,LOW);
  (void)SPIMSTXSP(0x01A); 
  (void)SPIMSTXSP(0x01);   //　ここまでがリセット
  (void)SPIMSTXSP(0x01C);
  (void)SPIMSTXSP(0xFF);   //　ここまでが直後のステータスのチェック
  digitalWrite(10,HIGH);
  delay(300);
  digitalWrite(10,LOW);     //　ここまでがCSBをHIGHにする
  (void)SPIMSTXSP(0x01C);
  (void)SPIMSTXSP(0xFF);    //　ここまでがステータスのチェック　リセット処理が完了している

  digitalWrite(10, LOW);
  delay(300);
  (void)SPIMSTXSP(0x16);   // SCPの初期化
  (void)SPIMSTXSP(0x01);   // RSTRレジスタに01を書き込むとリセット状態になります
  (void)SPIMSTXSP(0x16);
  (void)SPIMSTXSP(0x00);   // リセット状態から抜け出し初期化の処理を行います
  (void)SPIMSTXSP(0x18);   // これは何を意味しているの？
  (void)SPIMSTXSP(0xFF); 
  delay(300);
  (void)SPIMSTXSP(0x0E);   // 測定のオペレーション・モードの設定　測定は高精度モードを選択しますのでオペレーション・レジスタを選択し、高精度モードを設定する0x0Aのコードを書き込みます
  (void)SPIMSTXSP(0x0A);
  (void)SPIMSTXSP(0x10);   // オペレーションの実行完了を確認します。オペレーション・ステータス・レジスタを読み取るためのコード0x10をセットし送信します
  (void)SPIMSTXSP(0xFF);
  delay(300);
  (void)SPIMSTXSP(0x0C);   // オペレーション・モードの設定の後、オペレーション・レジスタを読み取り、オペレーション・モードの確認を行います。オペレーション・モードを読み取るためのコード、0x0Cをセットしています
  (void)SPIMSTXSP(0xFF);
}

void loop()
{
  double kiatsu, temp;
  long   rd0, rd1, rd2;

  // 気圧を取得
  (void)SPIMSTXSP(DATAREAD8_MSB);                     // まずは上位３ビットを取得
  rd0 = (long)(SPIMSTXSP(DUMMY_DATA) & 0x07) << 16;
  (void)SPIMSTXSP(DATAREAD16_LSB);                    // 次に、下位１６ビットを取得
  rd1 = rd0 + ((long)SPIMSTXSP(DUMMY_DATA) << 8);
  rd2 = rd1 + (long)SPIMSTXSP(DUMMY_DATA);
  Serial.println(rd2, HEX);                          // データを分かり易いように、１６進数で出力
  rd2 = rd2 & 0x07ffff;
  Serial.println(rd2, HEX);
  kiatsu = (double)(rd2) * 0.25 / 100.0;
  Serial.print(" kiatsu = ");
  Serial.println(kiatsu);
  // 温度を取得
  (void)SPIMSTXSP(TEMPOUT);
  rd1 = (long)SPIMSTXSP(DUMMY_DATA) << 8;
  rd2 = rd1 + (long)SPIMSTXSP(DUMMY_DATA);
  if(rd2 & 0x2000)rd2 = -(rd1 & 0x1fff);
  temp = (double)rd2 * 0.05;
  Serial.print(" temp = ");
  Serial.println(temp);
  delay(2000);                                          // 約２秒の待ち
}



